@@ -12,6 +12,10 @@ impl LikeFn<&DictArray> for DictEncoding {
         pattern: &dyn Array,
         options: LikeOptions,
     ) -> VortexResult<Option<ArrayRef>> {
+        // if we have more values than codes, it is faster to canonicalise first.
+        if array.values().len() > array.codes().len() {
+            return Ok(None);
+        }
         if let Some(pattern) = pattern.as_constant() {
             let pattern = ConstantArray::new(pattern, array.values().len()).into_array();
             let values = like(array.values(), &pattern, options)?;
