@@ -1,7 +1,6 @@
 use vortex_error::VortexResult;
 
 use crate::accessor::ArrayAccessor;
-use crate::arrays::varbin::stats::compute_is_constant;
 use crate::arrays::{VarBinArray, VarBinEncoding};
 use crate::compute::{IsConstantFn, IsConstantOpts};
 
@@ -14,3 +13,15 @@ impl IsConstantFn<&VarBinArray> for VarBinEncoding {
         array.with_iterator(compute_is_constant).map(Some)
     }
 }
+
+pub(super) fn compute_is_constant(iter: &mut dyn Iterator<Item = Option<&[u8]>>) -> bool {
+    let Some(first_value) = iter.next() else {
+        return false;
+    };
+    for v in iter {
+        if v != first_value {
+            return false;
+        }
+    }
+    true
+}
