@@ -2,6 +2,7 @@ use std::any::Any;
 
 use vortex_dtype::DType;
 use vortex_error::VortexResult;
+use vortex_mask::Mask;
 
 use crate::arrays::NullArray;
 use crate::builders::ArrayBuilder;
@@ -54,6 +55,12 @@ impl ArrayBuilder for NullBuilder {
         Ok(())
     }
 
+    fn ensure_capacity(&mut self, _capacity: usize) {}
+
+    fn set_validity(&mut self, validity: Mask) {
+        self.length = validity.len();
+    }
+
     fn finish(&mut self) -> ArrayRef {
         NullArray::new(self.length).into_array()
     }
