@@ -11,7 +11,7 @@ use crate::Array;
 use crate::compute::scalar_at;
 use crate::encoding::Encoding;
 
-#[derive(Debug, Copy, Clone)]
+#[derive(Debug, Copy, Clone, Eq, PartialEq)]
 pub enum SearchSortedSide {
     Left,
     Right,
@@ -27,9 +27,12 @@ impl Display for SearchSortedSide {
 }
 
 /// Result of performing search_sorted on an Array
+///
+/// See [`SearchSortedFn`] documentation for interpretation of the results
 #[derive(Debug, Copy, Clone, PartialEq, Eq)]
 pub enum SearchResult {
-    /// Result for a found element was found at the given index in the sorted array
+    /// Result for a found element was found in the array and another one could be inserted at the given position
+    /// in the sorted order
     Found(usize),
 
     /// Result for an element not found, but that could be inserted at the given position
@@ -58,11 +61,11 @@ impl SearchResult {
     ///
     /// For example for a ChunkedArray with chunk offsets array [0, 3, 8, 10] you can use this method to
     /// obtain index suitable for indexing into it after performing a search
-    pub fn to_offsets_index(self, len: usize) -> usize {
+    pub fn to_offsets_index(self, len: usize, side: SearchSortedSide) -> usize {
         match self {
             SearchResult::Found(i) => {
-                if i == len {
-                    i - 1
+                if side == SearchSortedSide::Right || i == len {
+                    i.saturating_sub(1)
                 } else {
                     i
                 }
@@ -80,18 +83,6 @@ impl SearchResult {
         let idx = self.to_index();
         if idx == len { idx - 1 } else { idx }
     }
-
-    /// Apply a transformation to the Found or NotFound index.
-    #[inline]
-    pub fn map<F>(self, f: F) -> SearchResult
-    where
-        F: FnOnce(usize) -> usize,
-    {
-        match self {
-            SearchResult::Found(i) => SearchResult::Found(f(i)),
-            SearchResult::NotFound(i) => SearchResult::NotFound(f(i)),
-        }
-    }
 }
 
 impl Display for SearchResult {
@@ -105,7 +96,12 @@ impl Display for SearchResult {
 
 /// Searches for value assuming the array is sorted.
 ///
-/// For nullable arrays we assume that the nulls are sorted last, i.e. they're the greatest value
+/// Returned indices satisfy following condition if the search for value was to be inserted into the array at found positions
+///
+/// |side |result satisfies|
+/// |-----|----------------|
+/// |left |array\[i-1\] < value <= array\[i\]|
+/// |right|array\[i-1\] <= value < array\[i\]|
 pub trait SearchSortedFn<A: Copy> {
     fn search_sorted(
         &self,
@@ -328,6 +324,7 @@ pub fn search_sorted_usize_many(
         .try_collect()
 }
 
+#[allow(clippy::len_without_is_empty)]
 pub trait IndexOrd<V> {
     /// PartialOrd of the value at index `idx` with `elem`.
     /// For example, if self\[idx\] > elem, return Some(Greater).
@@ -348,10 +345,8 @@ pub trait IndexOrd<V> {
     fn index_ge(&self, idx: usize, elem: &V) -> bool {
         matches!(self.index_cmp(idx, elem), Some(Greater | Equal))
     }
-}
 
-#[allow(clippy::len_without_is_empty)]
-pub trait Len {
+    /// Get the length of the underlying ordered collection
     fn len(&self) -> usize;
 }
 
@@ -399,7 +394,7 @@ pub trait SearchSorted<T> {
 // Default implementation for types that implement IndexOrd.
 impl<S, T> SearchSorted<T> for S
 where
-    S: IndexOrd<T> + Len + ?Sized,
+    S: IndexOrd<T> + ?Sized,
 {
     fn search_sorted_by<F: FnMut(usize) -> Ordering, N: FnMut(usize) -> Ordering>(
         &self,
@@ -486,6 +481,10 @@ impl IndexOrd<Scalar> for dyn Array + '_ {
         let scalar_a = scalar_at(self, idx).ok()?;
         scalar_a.partial_cmp(elem)
     }
+
+    fn len(&self) -> usize {
+        Self::len(self)
+    }
 }
 
 impl<T: PartialOrd> IndexOrd<T> for [T] {
@@ -493,16 +492,7 @@ impl<T: PartialOrd> IndexOrd<T> for [T] {
         // SAFETY: Used in search_sorted_by same as the standard library. The search_sorted ensures idx is in bounds
         unsafe { self.get_unchecked(idx) }.partial_cmp(elem)
     }
-}
 
-impl Len for dyn Array + '_ {
-    #[allow(clippy::same_name_method)]
-    fn len(&self) -> usize {
-        Self::len(self)
-    }
-}
-
-impl<T> Len for [T] {
     fn len(&self) -> usize {
         self.len()
     }
