@@ -3,7 +3,8 @@ use std::ops::Range;
 use std::os::unix::prelude::FileExt;
 use std::sync::Arc;
 
-use bytes::Bytes;
+use bytes::BytesMut;
+use futures::future::try_join_all;
 use futures_util::StreamExt;
 use object_store::path::Path;
 use object_store::{
@@ -113,31 +114,63 @@ impl VortexReadAt for ObjectStoreReadAt {
 
 pub struct ObjectStoreWriter {
     upload: Box<dyn MultipartUpload>,
+    buffer: BytesMut,
 }
 
+const CHUNKS_SIZE: usize = 25 * 1024 * 1024;
+
 impl ObjectStoreWriter {
     pub async fn new(object_store: Arc<dyn ObjectStore>, location: Path) -> VortexResult<Self> {
         let upload = object_store.put_multipart(&location).await?;
-        Ok(Self { upload })
+        Ok(Self {
+            upload,
+            buffer: BytesMut::with_capacity(CHUNKS_SIZE),
+        })
     }
 }
 
 impl VortexWrite for ObjectStoreWriter {
     async fn write_all<B: IoBuf>(&mut self, buffer: B) -> io::Result<B> {
-        const CHUNKS_SIZE: usize = 25 * 1024 * 1024;
+        self.buffer.extend_from_slice(buffer.as_slice());
 
-        for chunk in buffer.as_slice().chunks(CHUNKS_SIZE) {
-            let payload = Bytes::copy_from_slice(chunk);
-            self.upload
-                .as_mut()
-                .put_part(PutPayload::from_bytes(payload))
-                .await?;
+        if self.buffer.len() > CHUNKS_SIZE {
+            let mut buffer =
+                std::mem::replace(&mut self.buffer, BytesMut::with_capacity(CHUNKS_SIZE)).freeze();
+            let mut parts = vec![];
+
+            while buffer.len() > CHUNKS_SIZE {
+                let payload = buffer.split_to(CHUNKS_SIZE);
+                let part_fut = self
+                    .upload
+                    .as_mut()
+                    .put_part(PutPayload::from_bytes(payload));
+
+                parts.push(part_fut);
+            }
+
+            try_join_all(parts).await?;
         }
 
         Ok(buffer)
     }
 
     async fn flush(&mut self) -> io::Result<()> {
+        let mut buffer = std::mem::take(&mut self.buffer).freeze();
+        let mut parts = vec![];
+
+        while !buffer.is_empty() {
+            let chunk_size = usize::min(buffer.len(), CHUNKS_SIZE);
+            let payload = buffer.split_to(chunk_size);
+            let part_fut = self
+                .upload
+                .as_mut()
+                .put_part(PutPayload::from_bytes(payload));
+
+            parts.push(part_fut);
+        }
+
+        try_join_all(parts).await?;
+
         self.upload.complete().await?;
         Ok(())
     }
