@@ -12,7 +12,7 @@ use vortex_flatbuffers::{FlatBuffer, FlatBufferRoot, WriteFlatBuffer, layout};
 use crate::LayoutId;
 use crate::context::LayoutContext;
 use crate::reader::LayoutReader;
-use crate::segments::{AsyncSegmentReader, SegmentId};
+use crate::segments::{SegmentId, SegmentSource};
 use crate::vtable::LayoutVTableRef;
 
 /// [`Layout`] is the lazy equivalent to [`vortex_array::ArrayRef`], providing a hierarchical
@@ -102,10 +102,10 @@ impl Layout {
     }
 
     /// Returns the human-readable name of the layout.
-    pub fn name(&self) -> &str {
+    pub fn name(&self) -> &Arc<str> {
         match &self.0 {
-            Inner::Owned(owned) => owned.name.as_ref(),
-            Inner::Viewed(viewed) => viewed.name.as_ref(),
+            Inner::Owned(owned) => &owned.name,
+            Inner::Viewed(viewed) => &viewed.name,
         }
     }
 
@@ -244,7 +244,7 @@ impl Layout {
         (0..self.nsegments()).map(move |i| self.segment_id(i).vortex_expect("segment bounds"))
     }
 
-    /// Returns the layout metadata
+    /// Returns the bytes of the metadata stored in the layout's flatbuffer.
     pub fn metadata(&self) -> Option<Bytes> {
         match &self.0 {
             Inner::Owned(owned) => owned.metadata.clone(),
@@ -258,10 +258,10 @@ impl Layout {
     /// Create a reader for this layout.
     pub fn reader(
         &self,
-        segment_reader: Arc<dyn AsyncSegmentReader>,
-        ctx: ArrayContext,
-    ) -> VortexResult<Arc<dyn LayoutReader + 'static>> {
-        self.vtable().reader(self.clone(), ctx, segment_reader)
+        segment_source: &Arc<dyn SegmentSource>,
+        ctx: &ArrayContext,
+    ) -> VortexResult<Arc<dyn LayoutReader>> {
+        self.vtable().reader(self.clone(), segment_source, ctx)
     }
 
     /// Register splits for this layout.
